import asyncio

import flask

import chameleon_flask
import chameleon_flask as fc


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_friendly_404_sync_method(setup_global_template):
    @fc.template('home/index.pt')
    def view_method(a, b, c):
        chameleon_flask.not_found()
        return {'a': a, 'b': b, 'c': c}

    resp = view_method(1, 2, 3)
    html = response_to_html(resp)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 404
    assert '<h1>This is a pretty 404 page.</h1>' in html


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_friendly_404_custom_template_sync_method(setup_global_template):
    @fc.template('home/index.pt')
    def view_method(a, b, c):
        chameleon_flask.not_found(four04template_file='errors/other_error_page.pt')
        return {'a': a, 'b': b, 'c': c}

    resp = view_method(1, 2, 3)
    html = response_to_html(resp)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 404
    assert '<h1>Another pretty 404 page.</h1>' in html


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_friendly_404_async_method(setup_global_template):
    @fc.template('home/index.pt')
    async def view_method(a, b, c) -> flask.Response:
        chameleon_flask.not_found()
        return {'a': a, 'b': b, 'c': c}

    resp = asyncio.run(view_method(1, 2, 3))
    html = response_to_html(resp)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 404
    assert '<h1>This is a pretty 404 page.</h1>' in html


def response_to_html(response: flask.Response) -> str:
    # noinspection PyUnresolvedReferences
    return response.response[0].decode('utf-8')


import pytest

import chameleon_flask as fc
from chameleon_flask.exceptions import FlaskChameleonException


def test_cannot_decorate_with_missing_init():
    fc.engine.clear()

    with pytest.raises(FlaskChameleonException):

        @fc.template('home/index.pt')
        def view_method(a, b, c):
            return {'a': a, 'b': b, 'c': c}

        view_method(1, 2, 3)


def test_can_call_init_with_good_path(test_templates_path):
    fc.global_init(str(test_templates_path), cache_init=False)

    # Clear paths to no effect future tests
    fc.engine.clear()


def test_cannot_call_init_with_bad_path(test_templates_path):
    bad_path = test_templates_path / 'missing'
    with pytest.raises(Exception):
        fc.global_init(str(bad_path), cache_init=False)



"""chameleon-flask - Adds integration of the Chameleon template language to Flask and Quart."""

__version__ = '0.0.2'
__author__ = 'Michael Kennedy <michael@talkpython.fm>'
__all__ = ['template', 'global_init', 'not_found', 'response', ]

from .engine import global_init
from .engine import not_found
from .engine import response
from .engine import template



import asyncio

import flask
# noinspection PyPackageRequirements
import pytest

import chameleon_flask as fc


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_cannot_decorate_missing_template(setup_global_template):
    with pytest.raises(ValueError):

        @fc.template('home/missing.pt')
        def view_method():
            return {}

        view_method()


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_requires_template_for_default_name(setup_global_template):
    with pytest.raises(ValueError):

        @fc.template(None)
        def view_method():
            return {}

        view_method()


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_default_template_name_pt(setup_global_template):
    @fc.template()
    def index(a, b, c):
        return {'a': a, 'b': b, 'c': c, 'world': 'WORLD'}

    resp = index(1, 2, 3)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 200
    html = response_to_html(resp)
    assert '<h1>Hello default WORLD!</h1>' in html


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_default_template_name_no_parentheses(setup_global_template):
    @fc.template
    def index(a, b, c):
        return {'a': a, 'b': b, 'c': c, 'world': 'WORLD'}

    resp = index(1, 2, 3)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 200
    html = response_to_html(resp)
    assert '<h1>Hello default WORLD!</h1>' in html


def test_default_template_name_html(setup_global_template):
    @fc.template()
    def details(a, b, c):
        return {'a': a, 'b': b, 'c': c, 'world': 'WORLD'}

    resp = details(1, 2, 3)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 200
    html = response_to_html(resp)
    assert '<h1>Hello default WORLD!</h1>' in html


# setup_global_template - needed as pytest mix-in.
# noinspection PyUnusedLocal
def test_can_decorate_dict_sync_method(setup_global_template):
    @fc.template('home/index.pt')
    def view_method(a, b, c):
        return {'a': a, 'b': b, 'c': c}

    resp = view_method(1, 2, 3)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 200


def test_can_decorate_dict_async_method(setup_global_template):
    @fc.template('home/index.pt')
    async def view_method(a, b, c):
        return {'a': a, 'b': b, 'c': c}

    resp = asyncio.run(view_method(1, 2, 3))
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 200


def test_direct_response_pass_through():
    @fc.template('home/index.pt')
    def view_method(a, b, c):
        return flask.Response(response='abc', status=418)

    resp = view_method(1, 2, 3)
    html = response_to_html(resp)
    assert isinstance(resp, flask.Response)
    assert resp.status_code == 418
    assert html == 'abc'


def response_to_html(response: flask.Response) -> str:
    # noinspection PyUnresolvedReferences
    return response.response[0].decode('utf-8')